// Arduino Code for 4-button Radio Transmitter using ASK and 433MHz transmitter.

#include <RH_ASK.h>
#include <SPI.h>  // Not actually used, but needed to compile with RadioHead library

RH_ASK rf_driver;  // Initialize the ASK driver

const int buttonA = 2;
const int buttonB = 3;
const int buttonC = 4;
const int buttonD = 5;

bool flag = 0;

unsigned long buttonPressTime;  // Variable to store the millis() value when the button is pressed
String keyString = "123";       // Initial secret key

void setup() {
  Serial.begin(9600);
  pinMode(buttonA, INPUT_PULLUP);
  pinMode(buttonB, INPUT_PULLUP);
  pinMode(buttonC, INPUT_PULLUP);
  pinMode(buttonD, INPUT_PULLUP);

  // Initialize RF driver
  if (!rf_driver.init()) {
    Serial.println("RF Driver initialization failed!");
  }
}

void loop() {
  String msg = "";

  // Check if button A and B are pressed together to generate a new key
  if ((digitalRead(buttonA) == LOW) && (digitalRead(buttonB) == LOW)) {
    keyString = generateKey();
    Serial.print("New key generated: ");
    Serial.println(keyString);
  }
  // Otherwise, handle individual button presses
  else if (digitalRead(buttonA) == LOW) {
    flag = 1;
    msg = keyString + "A";
  } else if (digitalRead(buttonB) == LOW) {
    flag = 1;
    msg = keyString + "B";
  } else if (digitalRead(buttonC) == LOW)  {
    flag = 1;
    msg = keyString + "C";
  } else if (digitalRead(buttonD) == LOW){
    flag = 1;
    msg = keyString + "D";
  } else if ((digitalRead(buttonD) == HIGH) && (flag == 1))  {
    msg = keyString + "E";
    flag = 0; // flag to stop constantly sending "E's" over the radio spectrum ;-)
  }

  // Send the message if any button is pressed
  if (msg.length() > 0) {
    rf_driver.send((uint8_t *)msg.c_str(), msg.length());  // Specify the length explicitly
    rf_driver.waitPacketSent();
    Serial.print("Message sent: ");
    Serial.println(msg);
  }

  delay(50);  // Short debounce delay
}

// Function to generate a new 3-letter key based on millis() value
String generateKey() {

  // Read and store the current millis value
  buttonPressTime = millis();

  // Convert the millis value to a string
  String key = String(buttonPressTime);

  // Ensure the key is exactly 3 characters long
  if (key.length() > 3) {
    // If longer than 3 characters, truncate it
    key = key.substring(0, 3);
  } else {
    // If shorter than 3 characters, pad it with leading zeros
    while (key.length() < 3) {
      key = "0" + key;
    }
  }

  // Send the new key to the receiver 10 times
  for (int i = 0; i < 10; i++) {
    // Send exactly 3 characters from the key
    rf_driver.send((uint8_t *)key.c_str(), 3);
    rf_driver.waitPacketSent();
    delay(1000);
  }
  key.remove(key.length() - 4); // pruning the data because Arduino compiler seems to add null character to string and I don't now why!! 
  return key;
}
